package djmil.cordacheckers.contracts;

import java.util.List;
import java.util.Optional;

import djmil.cordacheckers.states.GameState;
import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;

public class UtxoLedgerTransactionUtil {
    public static <T extends GameCommand> T getSingleCommand(UtxoLedgerTransaction utxoTrx, Class<T> clazz) {
        return single(utxoTrx.getCommands(clazz), clazz);
    }

    public static <T extends GameState> T getSingleInputState(UtxoLedgerTransaction utxoTrx, Class<T> clazz) {
        return single(utxoTrx.getInputStates(clazz), clazz);
    }

    public static <T extends GameState> T getSingleOutputState(UtxoLedgerTransaction utxoTrx, Class<T> clazz) {
        return single(utxoTrx.getOutputStates(clazz), clazz);
    }

    private static <T> T single(List<T> list, Class<T> clazz) {
        return optional(list, clazz)
            .orElseThrow( () -> new IllegalStateException(NO_INSTANCES_OF +clazz.getName()) );
    }

    private static <T> Optional<T> optional(List<T> list, Class<T> clazz) {
        return list
            .stream()
            .reduce((a, b) -> {throw new IllegalStateException(MULTIPLE_INSTANCES_OF +clazz.getName());});
    }

    private static String MULTIPLE_INSTANCES_OF = "Multiple instances of ";
    private static String NO_INSTANCES_OF = "No instances of ";
}
