package djmil.cordacheckers.gamestate;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.contracts.GameInfo;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.InitiatedBy;
import net.corda.v5.application.flows.ResponderFlow;
import net.corda.v5.application.membership.MemberLookup;
import net.corda.v5.application.messaging.FlowSession;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.base.exceptions.CordaRuntimeException;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.UtxoLedgerService;
import net.corda.v5.ledger.utxo.transaction.UtxoLedgerTransaction;
import net.corda.v5.ledger.utxo.transaction.UtxoTransactionValidator;

@InitiatedBy(protocol = "gamestate-commit")
public class CommitTrxResponder implements ResponderFlow {
    
    private final static Logger log = LoggerFactory.getLogger(CommitTrxResponder.class);

    @CordaInject
    public MemberLookup memberLookup;

    @CordaInject
    public UtxoLedgerService utxoLedgerService;
    
    @Suspendable
    @Override
    public void call(FlowSession session) {
        UtxoTransactionValidator txValidator = trxToValidate -> {
            try {
                checkParticipants(session, trxToValidate);

                /*
                 * Other checks / actions ?
                 */

                log.info("Approval for " + trxToValidate.getId());
            } catch (Exception e) {
                log.warn("Validation has failed. " +e.getMessage());
                // We have to re-trow an exception to indicate validation failure
                throw new CordaRuntimeException(e.getClass().getCanonicalName(), e.getMessage(), e.getCause());
            }
        };

        try {
            this.utxoLedgerService
                .receiveFinality(session, txValidator);
        } catch (Exception e) {
            // Log Corda's specific message about finality validation failure
            log.warn(e.getMessage()); 
        }
    }

    @Suspendable
    void checkParticipants(FlowSession session, UtxoLedgerTransaction gameStateUtxo) throws ParticipantException {
        System.out.println("commit responder");
        final GameInfo info = new GameInfo(gameStateUtxo);
        final var conterpartyName = session.getCounterparty();
        if (info.issuer.compareTo(conterpartyName) != 0)
            throw new ParticipantException("Issuer", conterpartyName, info.issuer);

        final var myName = memberLookup.myInfo().getName();
         System.out.println("  my name" + myName + " opponentName "+ conterpartyName);
        if (myName.getOrganizationUnit().equals("Custodian"))
            return; // Custodian shall not validate state's counterparty

        final var opponentName = info.state.getOpponentName(myName); // throws NotInvolved
        if (conterpartyName.compareTo(opponentName) != 0)
            throw new ParticipantException("Opponent", conterpartyName, opponentName);
    }

    public static class ParticipantException extends Exception {
        public ParticipantException(String role, MemberX500Name expected, MemberX500Name actual) {
            super("Bad participants. " +role +" role: expected '" +expected +"', actual '" +actual +"'");
        }
    }

}
